/* $BEGIN_LICENSE

This file is part of Minitube.
Copyright 2009, Flavio Tordini <flavio.tordini@gmail.com>

Minitube is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Minitube is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Minitube.  If not, see <http://www.gnu.org/licenses/>.

$END_LICENSE */

#ifndef CHANNELITEMDELEGATE_H
#define CHANNELITEMDELEGATE_H

#include <QtWidgets>

class ChannelItemDelegate : public QStyledItemDelegate {

    Q_OBJECT

public:
    ChannelItemDelegate(QObject* parent = 0);
    QSize sizeHint(const QStyleOptionViewItem&, const QModelIndex&) const;
    void paint(QPainter*, const QStyleOptionViewItem&, const QModelIndex&) const;

private:
    void paintChannel(QPainter*, const QStyleOptionViewItem&, const QModelIndex&) const;
    void paintAggregate(QPainter*, const QStyleOptionViewItem&, const QModelIndex&) const;
    void paintUnwatched(QPainter*, const QStyleOptionViewItem&, const QModelIndex&) const;
    void paintBadge(QPainter *painter, const QRect &line, const QString &text) const;
    void drawItem(QPainter *painter, const QRect &line,
                  const QPixmap &thumbnail, const QString &name) const;

};

#endif // CHANNELITEMDELEGATE_H
