QMainWindow > QToolBar QToolButton::menu-indicator { image: none; }

QStatusBar {
    border: 0;
    padding: 0;
    background: palette(base);
    color: palette(text);
}

QStatusBar QToolBar {
    padding:0;
    spacing:0;
    margin:0;
    border:0;
}

QStatusBar::item {
    border: 0;
    padding: 0;
    margin: 0;
}

QStatusBar QToolButton::menu-indicator {
    image: none;
}

AppsWidget {
    background-color: palette(window);
    border-top: 1px solid palette(mid);
}

RegionsView QPushButton[regionId] {
    margin: 5px;
    padding: 7px 10px;
    text-align: left;
    vertical-align: middle;
    background-color: transparent;
    border-radius: 3px;
}

RegionsView QPushButton[regionId]:hover, RegionsView QPushButton[regionId]:checked {
    background: palette(highlight);
    color: palette(highlighted-text);
}

RegionsView QPushButton[regionId]:focus {
    background: palette(mid);
    outline: 0;
}

SidebarHeader {
    padding: 0;
    margin: 0;
    spacing: 0;
    border: 0;
}

SidebarHeader QToolButton {
    border: 0;
    padding: 0;
    margin: 0;
    spacing: 0;
    height: 20;
    width: 24;
}

SidebarHeader QPushButton {
    background: transparent;
    text-align: center;
}

SidebarHeader QComboBox::drop-down {
    width: 0;
    border-style: none;
}

SearchView *[recentItem="true"], SearchView *[recentHeader="true"] {
    border: 0;
    padding: 4px 0;
}

SearchView *[recentItem="true"]:focus {
    outline: 1px solid palette(highlight);
}

MessageBar {
    background: #f9f7c8;
    border-radius: 5px;
}

MessageBar QLabel {
    color: rgba(0,0,0,192);
}

MessageBar QToolButton {
    background: transparent;
    border: 0;
}
